@@ -54,18 +54,23 @@ SettingsDialog::SettingsDialog(QSettings* pSettings, QWidget* pParent)
     m_pPreferredBitrateSelector->setModel(pBitrateListModel);
     m_PreferredBitrateBtn.setValueLayout(QMaemo5ValueButton::ValueUnderTextCentered); // Maemo5 style guide wants the menu entries centered.
     m_PreferredBitrateBtn.setPickSelector(m_pPreferredBitrateSelector);
+    m_pPreferredSyncBitrateSelector = new QMaemo5ListPickSelector(this);
+    m_pPreferredSyncBitrateSelector->setModel(pBitrateListModel);
+    m_PreferredSyncBitrateBtn.setValueLayout(QMaemo5ValueButton::ValueUnderTextCentered); // Maemo5 style guide wants the menu entries centered.
+    m_PreferredSyncBitrateBtn.setPickSelector(m_pPreferredSyncBitrateSelector);
 
 #else
 
     m_PreferredBitrateCombo.addItems(m_bitrateList);
+    m_PreferredSyncBitrateCombo.addItems(m_bitrateList);
 
 #endif
 
     //set up UI language value button / selector
 
-    m_UILangList << "English" << "Suomi" << "Dutch" << "German"; //possible UI language values
+    m_UILangList << "English" << "Suomi" << "Dutch" << "German" << "French"; //possible UI language values
 
-    m_UILangCodeList << "en" << "fi" << "nl" << "de"; //corresponding internal language codes
+    m_UILangCodeList << "en" << "fi" << "nl" << "de" << "fr"; //corresponding internal language codes
 
 #ifdef Q_WS_MAEMO_5
 
@@ -107,8 +112,6 @@ SettingsDialog::SettingsDialog(QSettings* pSettings, QWidget* pParent)
 
 #endif
 
-    m_PlaylistLoadingTOInput.setInputMask("900");
-
     m_LastFMUserIDInput.setInputMethodHints(Qt::ImhNoAutoUppercase);
     m_LastFMPasswordInput.setEchoMode(QLineEdit::Password); //set password mode for input field
 
@@ -120,26 +123,24 @@ SettingsDialog::SettingsDialog(QSettings* pSettings, QWidget* pParent)
 
     m_ScrobblingOptions.setLayout(&m_ScrobblingOptionLayout);
 
-    m_PlaylistOptionLayout.addWidget(&m_PlaylistLoadingTOCaption,0,0,1,1);
-    m_PlaylistOptionLayout.addWidget(&m_PlaylistLoadingTOInput,0,1,1,1);
-
-    m_PlaylistOptions.setLayout(&m_PlaylistOptionLayout);
-
     m_GridLayout.addWidget(&m_SearchOptions,0,0,1,-1);
     m_GridLayout.addWidget(&m_BufferingOptions,1,0,1,-1);
     m_GridLayout.addWidget(&m_HideCoverArt,2,0,1,-1);
 #ifdef Q_WS_MAEMO_5
     m_GridLayout.addWidget(&m_PreferredBitrateBtn,3,0,1,-1);
-    m_GridLayout.addWidget(&m_UILangBtn,4,0,1,-1);
-    m_GridLayout.addWidget(&m_OrientationBtn,5,0,1,-1);
+    m_GridLayout.addWidget(&m_PreferredSyncBitrateBtn,4,0,1,-1);
+    m_GridLayout.addWidget(&m_AllowMobileSyncSelection,5,0,1,-1);
+    m_GridLayout.addWidget(&m_UILangBtn,6,0,1,-1);
+    m_GridLayout.addWidget(&m_OrientationBtn,7,0,1,-1);
 #else
     m_GridLayout.addWidget(&m_PreferredBitrateCombo,3,0,1,-1);
-    m_GridLayout.addWidget(&m_UILangCombo,4,0,1,-1);
-    m_GridLayout.addWidget(&m_OrientationCombo,5,0,1,-1);
+    m_GridLayout.addWidget(&m_PreferredSyncBitrateCombo,4,0,1,-1);
+    m_GridLayout.addWidget(&m_AllowMobileSyncSelection,5,0,1,-1);
+    m_GridLayout.addWidget(&m_UILangCombo,6,0,1,-1);
+    m_GridLayout.addWidget(&m_OrientationCombo,7,0,1,-1);
 #endif
-    m_GridLayout.addWidget(&m_PlaylistOptions,6,0,1,-1);
-    m_GridLayout.addWidget(&m_PauseOnHSDisconnSelection,7,0,1,-1);
-    m_GridLayout.addWidget(&m_ScrobblingOptions,8,0,1,-1);
+    m_GridLayout.addWidget(&m_PauseOnHSDisconnSelection,8,0,1,-1);
+    m_GridLayout.addWidget(&m_ScrobblingOptions,9,0,1,-1);
 
     m_MainAreaLayout.setLayout(&m_GridLayout);
     m_MainArea.setWidget(&m_MainAreaLayout);
@@ -170,11 +171,11 @@ void SettingsDialog::setCapsLandscape()
     m_PreBufferNextTrackSelection.setText(tr("Enable track pre-buffering"));
 #ifdef Q_WS_MAEMO_5
     m_PreferredBitrateBtn.setText(tr("Preferred bitrate"));
+    m_PreferredSyncBitrateBtn.setText(tr("Preferred offline sync. bitrate"));
     m_UILangBtn.setText(tr("UI Language"));
     m_OrientationBtn.setText(tr("Display orientation"));
 #endif
-    m_PlaylistOptions.setTitle(tr("Playlist options"));
-    m_PlaylistLoadingTOCaption.setText(tr("Playlist load timeout (secs)"));
+    m_AllowMobileSyncSelection.setText(tr("Allow sync. over 3G"));
     m_PauseOnHSDisconnSelection.setText(tr("Pause on headset unplugged"));
     m_ScrobblingOptions.setTitle(tr("Scrobbling options"));
     m_EnableScrobblingSelection.setText(tr("Enable scrobbling to Last.fm"));
@@ -198,11 +199,11 @@ void SettingsDialog::setCapsPortrait()
     m_PreBufferNextTrackSelection.setText(tr("Enable track pre-buff."));
 #ifdef Q_WS_MAEMO_5
     m_PreferredBitrateBtn.setText(tr("Preferred bitrate"));
+    m_PreferredSyncBitrateBtn.setText(tr("Preferred offline bitrate"));
     m_UILangBtn.setText(tr("UI Language"));
     m_OrientationBtn.setText(tr("Display orientation"));
 #endif
-    m_PlaylistOptions.setTitle(tr("Playlist options"));
-    m_PlaylistLoadingTOCaption.setText(tr("Playlist load TO (secs)"));
+    m_AllowMobileSyncSelection.setText(tr("Allow sync. over 3G"));
     m_PauseOnHSDisconnSelection.setText(tr("Pause on headset unplugged"));
     m_ScrobblingOptions.setTitle(tr("Scrobbling options"));
     m_EnableScrobblingSelection.setText(tr("Enable Last.fm scrobbling"));
@@ -256,6 +257,19 @@ void SettingsDialog::readSettings()
 #else
         m_PreferredBitrateCombo.setCurrentIndex(m_prefBitrate);
 #endif
+        m_prefSyncBitrate = m_pAppSettings->value("QSpot/PrefSyncBitrate",IDX_BITRATE_160k).toInt();
+#ifdef Q_WS_MAEMO_5
+        m_pPreferredSyncBitrateSelector->setCurrentIndex(m_prefSyncBitrate);
+#else
+        m_PreferredSyncBitrateCombo.setCurrentIndex(m_prefSynxBitrate);
+#endif
+
+        m_allowMobileSync = m_pAppSettings->value("QSpot/allowMobileSync",ALLOW_MOBILE_SYNC_DEFAULT).toInt();
+
+        if(m_allowMobileSync)
+            m_AllowMobileSyncSelection.setChecked(true);
+        else
+            m_AllowMobileSyncSelection.setChecked(false);
 
         m_currUILang = m_UILangCodeList.indexOf(m_pAppSettings->value("QSpot/language",LANG_DEFAULT).toString());
 #ifdef Q_WS_MAEMO_5
@@ -271,9 +285,6 @@ void SettingsDialog::readSettings()
         m_OrientationCombo.setCurrentIndex(m_currOrientation);
 #endif
 
-        m_playlistLoadingTO = m_pAppSettings->value("QSpot/playlistLoadingTO",PLAYLIST_LOADING_TO_DEFAULT).toInt();
-        m_PlaylistLoadingTOInput.setText(QString::number(m_playlistLoadingTO));
-
         m_pauseOnHsetUnplugged = m_pAppSettings->value("QSpot/pauseOnHsetUnplugged",PAUSE_ON_HEADSET_UNPLUG_DEFAULT).toInt();
         if(m_pauseOnHsetUnplugged)
             m_PauseOnHSDisconnSelection.setChecked(true);
@@ -359,6 +370,7 @@ void SettingsDialog::saveSettings()
         }
 
 
+        //streaming bitrate selection
 #ifdef Q_WS_MAEMO_5
         selBitrate = m_pPreferredBitrateSelector->currentIndex();
 #else
@@ -368,6 +380,31 @@ void SettingsDialog::saveSettings()
         if(selBitrate!=m_prefBitrate)
         {
             m_pAppSettings->setValue("QSpot/PrefBitrate",selBitrate);
+            settingsChanged = true;
+        }
+
+        //offline sync. bitrate selection
+#ifdef Q_WS_MAEMO_5
+        selBitrate = m_pPreferredSyncBitrateSelector->currentIndex();
+#else
+        selBitrate = m_PreferredSyncBitrateCombo.currentIndex();
+#endif
+
+        if(selBitrate!=m_prefSyncBitrate)
+        {
+            m_pAppSettings->setValue("QSpot/PrefSyncBitrate",selBitrate);
+            settingsChanged = true;
+        }
+
+        if(m_AllowMobileSyncSelection.isChecked() && !m_allowMobileSync)
+        {
+            m_pAppSettings->setValue("QSpot/allowMobileSync",1);
+            settingsChanged = true;
+        }
+        else if(!m_AllowMobileSyncSelection.isChecked() && m_allowMobileSync)
+        {
+            m_pAppSettings->setValue("QSpot/allowMobileSync",0);
+            settingsChanged = true;
         }
 
 #ifdef Q_WS_MAEMO_5
@@ -399,12 +436,6 @@ void SettingsDialog::saveSettings()
             settingsChanged = true;
         }
 
-        if(m_PlaylistLoadingTOInput.text().toInt()!=m_playlistLoadingTO)
-        {
-            m_pAppSettings->setValue("QSpot/playlistLoadingTO",m_PlaylistLoadingTOInput.text().toInt());
-            settingsChanged = true;
-        }
-
         if(m_PauseOnHSDisconnSelection.isChecked() && !m_pauseOnHsetUnplugged)
         {
             m_pAppSettings->setValue("QSpot/pauseOnHsetUnplugged",1);
