@@ -31,19 +31,18 @@
 
 //defines for default setting values
 
-#define MAX_TRACKS_DEFAULT  500
-#define MAX_ALBUMS_DEFAULT 10
-#define MAX_ARTISTS_DEFAULT 10
-#define MIN_BUFFER_LEN_DEFAULT  30
-#define PRE_BUFFERING_DEFAULT    1
-#define SAVE_CREDENTIALS_DEFAULT 0
-#define HIDE_COVERART_DEFAULT 0
-#define ENABLE_SCROBBLING_DEFAULT 0
-#define ORIENTATION_DEFAULT 0
-#define PLAYLIST_LOADING_TO_DEFAULT 30
-#define PAUSE_ON_HEADSET_UNPLUG_DEFAULT 0
-#define LANG_DEFAULT "en"
-#define SPOTIFY_RECONNECT_INTERVAL 60
+#define MAX_TRACKS_DEFAULT                  500
+#define MAX_ALBUMS_DEFAULT                  10
+#define MAX_ARTISTS_DEFAULT                 10
+#define MIN_BUFFER_LEN_DEFAULT              30
+#define PRE_BUFFERING_DEFAULT               1
+#define SAVE_CREDENTIALS_DEFAULT            0
+#define HIDE_COVERART_DEFAULT               0
+#define ENABLE_SCROBBLING_DEFAULT           0
+#define ORIENTATION_DEFAULT                 0
+#define PAUSE_ON_HEADSET_UNPLUG_DEFAULT     0
+#define LANG_DEFAULT                        "en"
+#define ALLOW_MOBILE_SYNC_DEFAULT           1
 
 
 class SettingsDialog : public QDialog
@@ -92,15 +91,19 @@ private:
 
     QMaemo5ListPickSelector *m_pPreferredBitrateSelector; //pick selector associated with preferred bitrate value button
 
+    QMaemo5ValueButton m_PreferredSyncBitrateBtn; //value type button for selecting the preferred offline sync. bitrate
+
+    QMaemo5ListPickSelector *m_pPreferredSyncBitrateSelector; //pick selector associated with preferred offline sync. bitrate value button
+
 #else
 
     QComboBox m_PreferredBitrateCombo; //combo for selecting current bitrate
 
+    QComboBox m_PreferredSyncBitrateCombo; //combo for selecting current offline sync. bitrate
+
 #endif
 
-    QGroupBox m_PlaylistOptions; //container for playlist related settings
-
-    QGridLayout m_PlaylistOptionLayout;
+    QCheckBox m_AllowMobileSyncSelection; //checkbox for selecting whether sync. over mobile (3G) should be allowed or not
 
     QGroupBox m_ScrobblingOptions; //container for settings related to Last.fm scrobbling
 
@@ -134,7 +137,7 @@ private:
 
     QStringList m_UILangCodeList; //internal list of corresponding UI language codes (e.g., en for English)
 
-    QCheckBox m_PreBufferNextTrackSelection; //selected if automatic buffering of next track should be activated
+    QCheckBox m_PreBufferNextTrackSelection; //selected if automatic buffering of next track(s) should be activated
 
     QStringList m_orientationList; //list of device orientation values
 
@@ -150,12 +153,9 @@ private:
 
 #endif
 
-    QLabel m_PlaylistLoadingTOCaption;
-    QLineEdit m_PlaylistLoadingTOInput; //timeout value for playlist loading (in secs)
-
     QCheckBox m_PauseOnHSDisconnSelection; //selection to enable automatic pausing of music playback in case headset is unplugged
 
-    QCheckBox m_EnableScrobblingSelection; //enable / disable scrobbling to Last.Fm
+    QCheckBox m_EnableScrobblingSelection; //enable / disable scrobbling to Last.fm
 
     QLabel m_LastFMUserIDCaption;
     QLineEdit m_LastFMUserIDInput;  //Last.fm User ID input
@@ -168,31 +168,19 @@ private:
     //members holding current settings
 
     qint32 m_maxSearchTracks;
-
     qint32 m_maxSearchAlbums;
-
     qint32 m_maxSearchArtists;
-
     qint32 m_minBufferLen;
-
     qint32 m_preBufferingActive;
-
     qint32 m_hideCoverart;
-
     qint32 m_prefBitrate;
-
+    qint32 m_prefSyncBitrate;
+    qint32 m_allowMobileSync;
     qint32 m_currOrientation;
-
-    qint32 m_playlistLoadingTO;
-
     qint32 m_pauseOnHsetUnplugged;
-
     qint32 m_enableScrobbling;
-
     qint32 m_currUILang;
-
     QString m_strLastFMUserID;
-
     QString m_strLastFMPassword;
 
     void saveSettings(); //save (modified) settings
