@@ -8,30 +8,41 @@ MainWindow::MainWindow(QWidget *parent)
     setAttribute(Qt::WA_Maemo5StackedWindow); //window is part of window stack
 #endif
 
-    QCoreApplication::setApplicationName("QSpot"); //Setup app name
+    QCoreApplication::setApplicationName("QSpot"); //Setup app name (needs to be FMRadio in order to override silent profile)
     QCoreApplication::setApplicationVersion(APP_VERSION); //Setup app version
 
     m_pAppSettings = new QSettings("qspot","qspot", this); //allocate application settings instance
+    if(m_pAppSettings->contains("QSpot/SaveCredentials"))
+    { //removed unused settings entries
+        m_pAppSettings->remove("QSpot/SaveCredentials");
+        m_pAppSettings->remove("QSpot/UserID");
+        m_pAppSettings->remove("QSpot/Password");
+    }
+
     setupUILang(); //setup the UI language
     setWindowTitle(tr("QSpot - Playlists"));
 
     m_playing = false;
     m_update = false;
-    m_reConnectReq = false;
 
     m_pSpectrum = new SpectrumAnalyzer(this,55,40);
     m_pSpectrum->setColor(QColor(255,255,255)); //set spectrum color to white
 
-    m_pSession = new Spotify::Session(this); //create new Spotify session instance
+    m_pSyncProgress = new QProgressBar(this); //allocate progressbar indicator (offline synchronisation status)
+    m_pSyncProgress->setVisible(false);
+    m_pSyncProgress->setMaximum(100);
+    m_pSyncProgress->setMinimum(0);
 
+    m_pSession = new Spotify::Session(this); //create new Spotify session instance
     m_pOssoWrapper = new OssoWrapper("QSpot", APP_VERSION, this); //create wrapper instance for libosso
 
 #ifdef Q_WS_MAEMO_5
-    if(m_pOssoWrapper)
+   if(m_pOssoWrapper)
     { //register / declare new playback object with OSSO libplayback (whitelisting support / support for playing in silent profile)
         m_pOssoWrapper->initOssoPlayback("QSpot");
         m_pOssoWrapper->requestPlaybackStateChange(PB_STATE_STOP); //request to enter initial stopped state
     }
+
 #endif
 
     m_pDBusWrapper = new DBusWrapper(this); //allocate D-bus interface wrapper
@@ -53,14 +64,6 @@ MainWindow::MainWindow(QWidget *parent)
     connect(m_pLastFMScrobbler,SIGNAL(updateNowPlayingStatus(bool,const QString&)),SLOT(OnScrobblingStatus(bool, const QString&)));
     connect(m_pLastFMScrobbler,SIGNAL(scrobbleTracksStatus(bool,const QString&)),SLOT(OnScrobblingStatus(bool, const QString&)));
 
-    m_pConnMan = new ConnectionManager(this); //allocate connection manager instance
-    connect(m_pConnMan,SIGNAL(reconnectRequired()),SLOT(OnSpotifyReconnectRequired()));
-
-    m_pTimeoutTimer  = new QTimer(this); //allocate timeout timer instance
-    connect(m_pTimeoutTimer,SIGNAL(timeout()),SLOT(OnPlaylistLoadingTimeout()));
-    m_pReconnectTimer = new QTimer(this); //allocate reconnect timer instance
-    connect(m_pReconnectTimer,SIGNAL(timeout()),SLOT(OnReconnectTimeout()));
-
     m_pHelpViewer = new HelpViewer(this); //allocate online help instance
 
     m_pLoginDlg = new LoginDialog(m_pSession, m_pAppSettings, this); //allocate login dialog instance
@@ -68,16 +71,24 @@ MainWindow::MainWindow(QWidget *parent)
     m_pSettingsDlg = new SettingsDialog(m_pAppSettings, this); //allocate settings dialog instance
 
     //set up the the main window menu
-    menuBar()->addAction(tr("Settings"));
-#ifdef Q_WS_MAEMO_5
-    menuBar()->addAction(tr("FM Transmitter"));
-#endif
-    menuBar()->addAction(tr("Online Help"));
-    menuBar()->addAction(tr("About"));
-    menuBar()->addAction(tr("Log Out"));
 
-    //setup the app About dialog box
-    initAboutDialog();
+    m_pSettingsAction = menuBar()->addAction(tr("Settings"));
+    m_pSettingsAction->setData(MENU_ACTION_SETTINGS);
+#ifdef Q_WS_MAEMO_5
+    m_pFMTransAction = menuBar()->addAction(tr("FM Transmitter"));
+    m_pFMTransAction->setData(MENU_ACTION_FM_TRANSMITTER);
+#endif
+    m_pHelpAction = menuBar()->addAction(tr("Online Help"));
+    m_pHelpAction->setData(MENU_ACTION_ONLINE_HELP);
+    m_pAboutAction = menuBar()->addAction(tr("About"));
+    m_pAboutAction->setData(MENU_ACTION_ABOUT);
+    m_pLogoutAction =  menuBar()->addAction(tr("Log Out"));
+    m_pLogoutAction->setData(MENU_ACTION_LOGOUT);
+    m_pOnlineStateAction = menuBar()->addAction(tr("Go Offline"));
+    m_pOnlineStateAction->setData(MENU_ACTION_OFFLINE_MODE);
+    m_pOnlineStateAction->setCheckable(true); //checkable action
+    m_pOnlineStateAction->setChecked(false);
+    connect(m_pOnlineStateAction,SIGNAL(triggered(bool)),SLOT(OnOfflineModeStateToggled(bool)));
 
     m_pTrackView = new TrackView(m_pSession,this);
     connect(m_pTrackView,SIGNAL(playQueueSet(const QList<Spotify::Track*>&,qint32)),SLOT(OnSetPlayQueue(const QList<Spotify::Track*>&,qint32)));
@@ -103,10 +114,11 @@ MainWindow::MainWindow(QWidget *parent)
 
     //init the playlist listview instance
     m_pPlaylistView = new QListView(this);
-    //m_pPlaylistView->setContextMenuPolicy(Qt::CustomContextMenu);
+    m_pPlaylistView->setContextMenuPolicy(Qt::CustomContextMenu);
     m_pPlaylistModel = new QStandardItemModel(this);
     m_pPlaylistDelegate = new ListViewDelegate(this);
     m_pPlaylistDelegate->setSubHeaderFontPointSize(14);
+    m_pPlaylistDelegate->setItemTextDisabledColor(QColor(190,190,190)); //set disabled item color (gray)
 
     m_pPlaylistView->setItemDelegate(m_pPlaylistDelegate); //connect the delegate to view
     m_pPlaylistView->setModel(m_pPlaylistModel); //connect the model to view.
@@ -118,15 +130,19 @@ MainWindow::MainWindow(QWidget *parent)
     connect(m_pSession,SIGNAL(loggedIn(Spotify::Error)),SLOT(OnLoginResponse(Spotify::Error)));
     connect(m_pSession,SIGNAL(loggedOut()),SLOT(OnLoggedOut()));
     connect(m_pSession,SIGNAL(playlistContainersReady(QList<Spotify::Playlist*>)),this,SLOT(OnPlaylistContainersReady(QList<Spotify::Playlist*>)));
+    connect(m_pSession,SIGNAL(playlistAdded(Spotify::Playlist*,qint32)),this,SLOT(OnPlaylistAdded(Spotify::Playlist*,qint32)));
+    connect(m_pSession,SIGNAL(playlistRemoved(qint32)),this,SLOT(OnPlaylistRemoved(qint32)));
     connect(m_pSession,SIGNAL(playing(bool)),SLOT(OnPlayingState(bool)));
     connect(m_pSession,SIGNAL(searchComplete(Spotify::Search*)),SLOT(OnSearchComplete(Spotify::Search*)));
     connect(m_pSession,SIGNAL(playbackPositionUpdated(qint64)),SLOT(OnPlaybackPositionUpdated(qint64)));
     connect(m_pSession,SIGNAL(playTokenLost()),SLOT(OnPlaybackTokenLost()),Qt::QueuedConnection);
+    connect(m_pSession,SIGNAL(offlineSyncStatusUpdated(bool, qint32)),SLOT(OnOfflineSyncStatusUpdated(bool, qint32)));
+    connect(m_pSession,SIGNAL(offlineSyncError(const QString&)),SLOT(OnOfflineSyncError(const QString&)));
+    connect(m_pSession,SIGNAL(offlineModeStateChanged(bool)),SLOT(OnOfflineModeStateChanged(bool)));
     connect(m_pPlaylistView,SIGNAL(activated(QModelIndex)),SLOT(OnPlaylistActivated(QModelIndex)));
     connect(m_pPlaylistView,SIGNAL(customContextMenuRequested(QPoint)),SLOT(OnContextMenuRequested(QPoint)));
     connect(menuBar(),SIGNAL(triggered(QAction*)),SLOT(OnMainMenuAction(QAction*)));
     connect(this,SIGNAL(showLogin()),SLOT(OnShowLogin()),Qt::QueuedConnection);
-    connect(this,SIGNAL(handleLoginType()),SLOT(OnHandleLoginType()),Qt::QueuedConnection);
     connect(&m_NowPlayingIcon,SIGNAL(clicked()),SLOT(OnNowPlaying()));
     connect(&m_NowPlayingInfo,SIGNAL(clicked()), SLOT(OnNowPlaying()));
     connect(&m_NowPlayingPositionInfo,SIGNAL(clicked()), SLOT(OnNowPlaying()));
@@ -145,7 +161,6 @@ MainWindow::MainWindow(QWidget *parent)
     m_pPlaylistModel->clear();
 
     QPixmap iconMap;
-
     iconMap.load("/opt/qspot/images/icon-nowplaying.png"); //load icon from file
 
     m_NowPlayingIcon.setPixmap(iconMap); //set now playing icon
@@ -164,67 +179,53 @@ MainWindow::MainWindow(QWidget *parent)
 
     m_GridLayout.addWidget(&m_SearchInput,0,0,1,3);
     m_GridLayout.addWidget(&m_SearchBtn,0,3);
-    m_GridLayout.addWidget(m_pPlaylistView,1,0,1,-1);
-    m_GridLayout.addWidget(&m_NowPlayingIcon,2,0);
-    m_GridLayout.addWidget(&m_NowPlayingInfo,2,1,1,1,Qt::AlignLeft);
-    m_GridLayout.addWidget(&m_NowPlayingPositionInfo,2,2,1,1,Qt::AlignRight);
-    m_GridLayout.addWidget(m_pSpectrum,2,3);
+    m_GridLayout.addWidget(m_pSyncProgress,1,0,1,-1);
+    m_GridLayout.addWidget(m_pPlaylistView,2,0,1,-1);
+    m_GridLayout.addWidget(&m_NowPlayingIcon,3,0);
+    m_GridLayout.addWidget(&m_NowPlayingInfo,3,1,1,1,Qt::AlignLeft);
+    m_GridLayout.addWidget(&m_NowPlayingPositionInfo,3,2,1,1,Qt::AlignRight);
+    m_GridLayout.addWidget(m_pSpectrum,3,3);
 
     m_MainLayout.setLayout(&m_GridLayout);
-
     setCentralWidget(&m_MainLayout);
 
-    m_AudioInit = false;
-    m_SessionInit = false;
-
     setDisplayOrientation(m_pAppSettings->value("QSpot/displayOrientation",ORIENTATION_DEFAULT).toInt()); //set initial display orientation for views / dialogs
 
-    emit handleLoginType(); //check whether we should login directly, or allow user to enter login credentials (display login dialog accordingly)
+    m_pSession->init(); //initialise new libspotify session prior to login
+
+    QTimer::singleShot(1500,this,SLOT(OnHandleLoginType())); //proceed to login process (timer event delays process so that any initialisation has time to finish)
 }
 
 void MainWindow::OnSpotifyLogin(const QString& strUserID, const QString& strPassword, bool saveCredentials)
 {
-    //Sends of actual login req. to Spotify service backend. Inits app network session if necessary.
+    //Sends of actual login request to Spotify service backend
+    Spotify::Error err;
 
-    Spotify::Error error;
-
-    if(!m_SessionInit)
-    {//session not initialised; init session
-        m_pSession->init(); //init new session prior to login
-        m_SessionInit = true;
-    }
-
-    if(m_pConnMan->init()) //init application network session (i.e., make sure that we have an active network connection before proceeding to login)
-    {
-        if(strUserID!="")
-        {//use credentials supplied via login dialog (manual login)
-            m_pSession->login(strUserID,strPassword,saveCredentials);
-        }
-        else
-        { //try to use remembered credentials (automatic login)
-            m_pSession->login(m_pAppSettings->value("QSpot/UserID","").toString(),m_pAppSettings->value("QSpot/Password","").toString(),saveCredentials);
-        }
+    if(strUserID!="")
+    {//use credentials supplied via login dialog (manual login). Stores user credentials in case saveCredentials set to true.
+        //reset possible credentials in settings file (not used)
+        m_pSession->login(strUserID,strPassword,saveCredentials);
     }
     else
-    {
-#ifdef Q_WS_MAEMO_5
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-            QMaemo5InformationBox::information(this,tr("Error:") + " " + tr("No suitable internet access point(s) found!"));
-#else
-            Banner::information(this,tr("Error:") + " " + tr("No suitable internet access point(s) found!");
-#endif
-        emit showLogin(); //emit signal requesting to show login dialog
+    { //try to use remembered credentials (automatic login)
+       err =  m_pSession->relogin();
+       if(err.isError())
+       {
+           qDebug() << "Relogin failed (no stored credentials)";
+           m_pLoginDlg->resetState();
+           emit showLogin(); //display login dialog
+       }
     }
 }
 
 void MainWindow::OnHandleLoginType()
 {
    //Slot for detecting / handling automatic / manual login to Spotify. Performs automatic-login in case user has specified that credentials should be saved;
-   //otherwise the login dialog is displayed so ther user can provide login credentials.
+   //otherwise the login dialog is displayed so the user can provide login credentials.
 
-    if(m_pAppSettings->value("QSpot/SaveCredentials",SAVE_CREDENTIALS_DEFAULT).toInt())
+    if(!m_pSession->getRememberedUser().isNull())
     {
-        //perform automatic login in case user has saved credentials
+        //perform automatic login in case user has stored credentials
 #ifdef Q_WS_MAEMO_5
         QMaemo5InformationBox::information(this,tr("Logging in to Spotify"));
 #else
@@ -237,7 +238,7 @@ void MainWindow::OnHandleLoginType()
     }
     else
     {
-        emit showLogin(); //emit signal requesting to show login dialog
+        emit showLogin(); //emit signal requesting to show login dialog (no stored login credentials)
     }
 }
 
@@ -248,18 +249,33 @@ void MainWindow::setupUILang()
     qApp->installTranslator(&m_appTranslator);
 }
 
-
-void MainWindow::initAboutDialog()
+void MainWindow::showAboutDialog()
 {
-    //init the application about dialog
+    //display the application about dialog
+
+    QString strOfflineTimeLeftFormatted = "----";
+    QTextStream out(&strOfflineTimeLeftFormatted);
+    qint32 daysLeft = 0;
+    qint32 hoursLeft = 0;
+    qint32 minsLeft  = 0;
+
+    if(m_pSession)
+    {    //retrieve time left until user is required to go online (for display)
+        if(m_pSession->getOfflineTimeLeft(daysLeft,hoursLeft,minsLeft))
+        {
+            strOfflineTimeLeftFormatted = "";
+            out << daysLeft << " " << tr("days") << " "  << hoursLeft << " " << tr("hours") << " " << minsLeft << " " << tr("minutes");
+        }
+    }
 
     m_AboutDlg.setWindowTitle(tr("About QSpot") + " " + QString(APP_VERSION));
     m_AboutDlg.setTopIcon("/usr/share/icons/hicolor/64x64/apps/qspot.png");
     m_AboutDlg.setBottomIcon("/opt/qspot/images/spotify-core-logo.png"); //powered by Spotify(R) Core
-    m_AboutDlg.setText(tr("An open-source Qt-based Spotify client for the N900/Maemo5.\n"
+    m_AboutDlg.setText(tr("An open-source Qt-based Spotify client for N900/Maemo5.\n"
                         "(C) 2011 Andreas Enbacka and The QSpot Team.\n\n"
-                        "Special thanks to Markus (markusm@tmo) for assistance with the graphical design and layout, "
-                        "and to Tomi (rantom@tmo), Florian (florian@tmo) and Benno (bartus@tmo) for the translations."));
+                        "Special thanks to Markus for assistance with the graphical design and layout, "
+                        "and to Tomi, Florian, Benno and Nicolas for the translations.\n\n") +
+                        tr("Go online within") + " " + strOfflineTimeLeftFormatted);
 
 
     QRect screenGeometry = QApplication::desktop()->screenGeometry(); // get current screen geometry
@@ -271,6 +287,8 @@ void MainWindow::initAboutDialog()
     {
         m_AboutDlg.setFixedSize(APP_HEIGHT,APP_WIDTH); //resize dialog to fixed size
     }
+
+    m_AboutDlg.exec();
 }
 
 MainWindow::~MainWindow()
@@ -304,6 +322,10 @@ void MainWindow::OnSettingsSaved()
 void MainWindow::updateSettings()
 {
    //update / set application-related settings
+
+   sp_bitrate selectedBitrate;
+   qint32 prefBitrateIdx;
+
     //buffering related settings
     m_pSession->setMinBufferSize(m_pAppSettings->value("QSpot/minBufferLen",MIN_BUFFER_LEN_DEFAULT).toInt()); //update min. buffer length (default to 30 secs)
 
@@ -326,7 +348,8 @@ void MainWindow::updateSettings()
         m_pQueueView->setPauseOnHSUnplugged(false);
 
     //scrobbling related settings
-    if(m_pAppSettings->value("QSpot/enableScrobbling",ENABLE_SCROBBLING_DEFAULT).toInt())
+    m_LastfmScrobblingEnabled = m_pAppSettings->value("QSpot/enableScrobbling",ENABLE_SCROBBLING_DEFAULT).toBool();
+    if(m_LastfmScrobblingEnabled)
     {
         //scrobbling enabled, send authentication request in order to retrieve a Last.fm session key
         m_pLastFMScrobbler->setCache(true); //enable caching of scrobbles
@@ -343,16 +366,44 @@ void MainWindow::updateSettings()
          m_pQueueView->setHideCoverart(false);
     }
 
-    //set preferred bitrate
-    sp_bitrate selectedBitrate;
-    int prefBitrate = m_pAppSettings->value("QSpot/PrefBitrate",IDX_BITRATE_160k).toInt();
-    if(prefBitrate==IDX_BITRATE_96k)
+    //set preferred streaming bitrate
+    prefBitrateIdx = m_pAppSettings->value("QSpot/PrefBitrate",IDX_BITRATE_160k).toInt();
+    if(prefBitrateIdx==IDX_BITRATE_96k)
         selectedBitrate = SP_BITRATE_96k;
-    else if(prefBitrate==IDX_BITRATE_320k)
+    else if(prefBitrateIdx==IDX_BITRATE_320k)
         selectedBitrate = SP_BITRATE_320k;
     else //default bitrate
         selectedBitrate = SP_BITRATE_160k;
-    m_pSession->setPreferredBitrate((selectedBitrate));
+    m_pSession->setPreferredBitrate(selectedBitrate);
+
+    //set preferred offline sync. bitrate
+    prefBitrateIdx = m_pAppSettings->value("QSpot/PrefSyncBitrate",IDX_BITRATE_160k).toInt();
+    if(prefBitrateIdx==IDX_BITRATE_96k)
+        selectedBitrate = SP_BITRATE_96k;
+    else if(prefBitrateIdx==IDX_BITRATE_320k)
+        selectedBitrate = SP_BITRATE_320k;
+    else //default bitrate
+        selectedBitrate = SP_BITRATE_160k;
+    m_pSession->setPreferredSyncBitrate(selectedBitrate);
+
+    //set whether offline sync. over mobile (e.g., 3G) connection should be allowed or not
+    if(m_pAppSettings->value("QSpot/allowMobileSync",ALLOW_MOBILE_SYNC_DEFAULT).toInt())
+    {
+        m_pSession->setAllowMobileSync(true);
+    }
+    else
+    {
+         m_pSession->setAllowMobileSync(false);
+    }
+
+
+    //check whether offline mode status should be changed
+    qint32 offlineMode = m_pAppSettings->value("QSpot/OfflineMode",FORCE_OFFLINE_MODE_OFF).toInt();
+    if(offlineMode && !m_pSession->isOfflineMode())
+    { //force offline mode
+       m_pSession->setOfflineMode(true,true);
+       m_pOnlineStateAction->setChecked(true); //set UI state
+    }
 }
 
 void MainWindow::OnShowLogin()
@@ -441,6 +492,24 @@ void MainWindow::OnSetPlayQueue(const QList<Spotify::Track*>& queue, qint32 curr
     //set the current (active) playqueue, and associated playback position within the queue.
     m_PlayQueue = queue;
     m_currQueuePos = currQueuePos;
+    if(sender()==m_pTrackView)
+    { //queue set from track view
+        if(m_pTrackView->getActivePlaylist())
+        {
+            if(m_pTrackView->getActivePlaylist()->getOfflineStatus()==Spotify::Available) //playlist is available offline
+                m_pQueueView->setQueueOffline(true); //queued tracks (playlist) available offline
+            else
+                m_pQueueView->setQueueOffline(false); //queued tracks (playlist) not available offline
+        }
+        else
+        {
+            m_pQueueView->setQueueOffline(false); //queued tracks (e.g., search results) not available offline
+        }
+    }
+    else //queue set from e.g., album view
+    {
+        m_pQueueView->setQueueOffline(false);
+    }
     m_pQueueView->setQueueInitialState(m_PlayQueue, currQueuePos);
     if(m_pArtistView->isVisible())
         m_pArtistView->hide();
@@ -492,15 +561,15 @@ void MainWindow::OnPlayQueuePosUpdated(qint32 pos)
 void MainWindow::logout()
 {//logout the curent user and display the login dialog
     //clear saved credentials
-    m_pAppSettings->setValue("QSpot/SaveCredentials",0);
-    m_pAppSettings->setValue("QSpot/UserID","");
-    m_pAppSettings->setValue("QSpot/Password","");
+    //m_pAppSettings->setValue("QSpot/SaveCredentials",0);
+   // m_pAppSettings->setValue("QSpot/UserID","");
+   // m_pAppSettings->setValue("QSpot/Password","");
+    m_pSession->forgetCurrentUser();
     m_pSession->logout(); //logout current user
 }
 
 void MainWindow::closeEvent(QCloseEvent *)
 {//event received upon application shutdown
-    //m_pSession->logout(); //logout current user
     m_pSession->logout(); //logout current user
 }
 
@@ -520,107 +589,115 @@ void MainWindow::OnLoginResponse(Spotify::Error error)
 #else
         Banner::information(this,tr("Error:") + error.toString());
 #endif
-        if(!m_reConnectReq)
-        {
-            m_pLoginDlg->resetState(); //re-enable login button etc.
-            emit showLogin(); //display login dialog in case auto-login is enabled and has failed
-        }
-        else
-        {
-            m_pReconnectTimer->start(SPOTIFY_RECONNECT_INTERVAL*1000); //start reconnect timer (new attempt to reconnect after SPOTIFY_RECONNECT_INTERVAL secs)
-        }
+        m_pLoginDlg->resetState(); //re-enable login button etc.
+        emit showLogin(); //display login dialog in case auto-login is enabled and has failed
     }
     else
     { //successful login to Spotify service
-        m_pReconnectTimer->stop(); //stop reconnect timer in case it is active
-        if(!m_reConnectReq) // do not reload playlist metadata on re-connect request
-        {
-            if(m_pLoginDlg->getSaveCredentials())
-            {//user credentials should be stored (for automatic re-login)
-                m_pAppSettings->setValue("QSpot/SaveCredentials",1);
-                m_pAppSettings->setValue("QSpot/UserID",m_pSession->getRememberedUser());
-                m_pAppSettings->setValue("QSpot/Password",m_pSession->getUserPwd());
-            }
-    #ifdef Q_WS_MAEMO_5
-            m_pLoginDlg->setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
-            setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display main window progress indicator
-    #endif
-            m_pLoginDlg->hide();
-            m_SearchInput.setFocus(); //set default focus to search field
-            m_pTimeoutTimer->start(m_pAppSettings->value("QSpot/playlistLoadingTO",PLAYLIST_LOADING_TO_DEFAULT).toInt()*1000); //set timeout value (in msecs)
-            m_pSession->getPlaylistContainer(); //request user playlist / track data (in case initial login request)
-    #ifdef Q_WS_MAEMO_5
-            QMaemo5InformationBox::information(this,tr("Loading playlists")); //display loading notification
-    #else
-            Banner::information(this,tr("Loading playlists")); //display loading notification
-    #endif
+#ifdef Q_WS_MAEMO_5
+        m_pLoginDlg->setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //hide progress indicator
+        setAttribute(Qt::WA_Maemo5ShowProgressIndicator,true); //display main window progress indicator
+#endif
+        m_pLogoutAction->setText(tr("Log Out") + " " + m_pSession->getCurrentUser()); //setup the Log Out menubar entry
+        m_pLoginDlg->hide();
+        m_SearchInput.setFocus(); //set default focus to search field
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this,tr("Loading playlists")); //display loading notification
+#else
+        Banner::information(this,tr("Loading playlists")); //display loading notification
+#endif
+    }
+}
+
+ void MainWindow::OnPlaylistAdded(Spotify::Playlist* playlist, qint32 position)
+ {
+     //a playlist has been added
+     if(playlist)
+     {   //connect updated signal
+         connect(playlist,SIGNAL(updated()),this,SLOT(OnPlaylistUpdated()));
+         m_Playlists.insert(position,playlist); //insert into corresponding pos in list
+         updatePlaylistModel(); //update playlists model
+     }
+ }
+
+  void MainWindow::OnPlaylistRemoved(qint32 position)
+  {
+      //a playlist has been removed
+
+      if(position<m_Playlists.size())
+      {
+          Spotify::Playlist* playlist = m_Playlists.takeAt(position);
+          if(playlist)
+          {
+              disconnect(playlist,SIGNAL(updated()),this,SLOT(OnPlaylistUpdated())); //disconnect signal / slot
+              updatePlaylistModel(); //update playlists model
+          }
+      }
+
+  }
+
+void MainWindow::OnPlaylistUpdated()
+{
+    //a playlist (e.g., metadata) has been updated
+    qDebug() << "Playlist updated..";
+
+    Spotify::Playlist* pPlaylist = (Spotify::Playlist*)sender(); //sender playlist instance
+    if(pPlaylist)
+        pPlaylist->load(); //re-load metadata
+
+    updatePlaylistModel(); //force model update (for changes to be e.g., reflected in playlists view)
+}
+
+void MainWindow::updatePlaylistModel()
+{
+    //update the listmodel from underlying playlist instances
+
+    QPixmap iconMap;
+    iconMap.load("/opt/qspot/images/icon-type-list.png"); //load playlist type icon from file
+    QIcon listTypeIcon(iconMap);
+    iconMap.load("/opt/qspot/images/icon-type-list-offline.png"); //load playlist type (offline) icon from file
+    QIcon listTypeOfflineIcon(iconMap);
+
+    m_pPlaylistModel->clear();
+
+    for(qint32 pos=0; pos < m_Playlists.size(); pos++)
+    {
+        Spotify::Playlist* pl = m_Playlists.at(pos);
+        QStandardItem *item = new QStandardItem();
+        item->setEditable(false);
+        if(pl->getOfflineStatus()!=Spotify::Available) //playlist not offline
+            item->setData(listTypeIcon,ListViewDelegate::iconRole);
+        else
+            item->setData(listTypeOfflineIcon,ListViewDelegate::iconRole);  //offline playlist
+        item->setData(pl->getName(),ListViewDelegate::headerRole);
+        item->setData(QString::number(pl->getNumTracks()) + " " + tr("tracks"),ListViewDelegate::subHeaderLeftJustifyRole);
+        if(pl->getOfflineStatus() != Spotify::Available && m_pSession->isOfflineMode())
+        {//we are currently in offline mode, and the playlist has not been synced for offline usage
+            item->setEnabled(false);
+            item->setData(-1,ListViewDelegate::customDataRole);
         }
         else
         {
-            m_reConnectReq = false;
-            m_pSession->pauseResumeBuffering(); //continue buffering in case it has been in progress before reconnect
+            item->setEnabled(true);
+            item->setData(pos,ListViewDelegate::customDataRole); //store associated position in playlist instance list
         }
+        m_pPlaylistModel->appendRow(item);
     }
 }
 
-void MainWindow::OnPlaylistLoadingTimeout()
-{
-    //Timeout has occured during playlist loading, request to interrupt the loading process
-
-    qDebug() << "Playlist loading timeout!";
-
-    m_pTimeoutTimer->stop();
-
-    if(m_pSession->playlistLoadingInProgress())
-    {
-        m_pSession->stopPlaylistLoading(); //interrupt playlist loading
-    }
-#ifdef Q_WS_MAEMO_5
-    QMaemo5InformationBox::information(this,tr("Timeout occured while loading playlists!")); //display timeout notification
-    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false); //disable progress indicator
-#else
-    Banner::information(this,tr("Timeout occured while loading playlists!")); //display timeout notification
-#endif
-
-    setupInitialState(); //setup the app initial state
-}
-
-void MainWindow::OnReconnectTimeout()
-{
-    //triggered by timer when automatic (timed) reconnect request should be performed (triggered once a minute)
-    m_pSession->relogin(); //try to re-connect to Spotify using current session credentials
-    m_pReconnectTimer->stop(); //stop the timer
-}
-
 void MainWindow::OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists)
 {
-   //add items to playlists list widget
-
-   QPixmap iconMap;
-
-   m_pTimeoutTimer->stop(); //stop timeout timer
-
-   iconMap.load("/opt/qspot/images/icon-type-list.png"); //load playlist type icon from file
-
-   QIcon listTypeIcon(iconMap);
+   //add items to playlists list widget (all user's playlists have successfully loaded)
 
    m_Playlists = playlists; //store list of playlists
-
-   m_pPlaylistModel->clear();
-
-   for(int pos=0; pos < playlists.size(); pos++)
+   for(qint32 pos=0; pos < m_Playlists.size(); pos++)
    {
-       Spotify::Playlist* pl = playlists.at(pos);
-       QStandardItem *item = new QStandardItem();
-       item->setEditable(false);
-       item->setData(listTypeIcon,ListViewDelegate::iconRole);
-       item->setData(pl->getName(),ListViewDelegate::headerRole);
-       item->setData(QString::number(pl->getNumTracks()) + " " + tr("tracks"),ListViewDelegate::subHeaderLeftJustifyRole);
-       item->setData(pos,Qt::UserRole); //store associated position in playlist instance list
-       m_pPlaylistModel->appendRow(item);
+       //setup signal / slot connections (for notification of playlist metadata / state updated)
+       Spotify::Playlist* pl = m_Playlists.at(pos);
+       connect(pl,SIGNAL(updated()),this,SLOT(OnPlaylistUpdated()));
    }
-
-   setupInitialState();
+   updatePlaylistModel(); //update listview data model
+   setupInitialState(); //setup initial state
 
 #ifdef Q_WS_MAEMO_5
    setAttribute(Qt::WA_Maemo5ShowProgressIndicator,false);
@@ -631,15 +708,14 @@ void MainWindow::setupInitialState()
 {
     //set up the app initial state (audio etc.)
 
-    if(!m_AudioInit)
-    {
-       m_pSession->initAudio(); //init audio interface
-       m_AudioInit = true;
-    }
+    m_pSession->initAudio(); //init audio interface
 
-    m_SearchInput.setDisabled(false);
+    if(!m_pSession->isOfflineMode())
+    {
+        m_SearchInput.setDisabled(false);
+        m_SearchBtn.setDisabled(false);
+    }
     m_SearchInput.setFocus(); //set initial (default) focus to the Search input field
-    m_SearchBtn.setDisabled(false);
 
     updateSettings(); //set application-related settings
 }
@@ -687,49 +763,50 @@ void MainWindow::OnPlaylistActivated(QModelIndex index)
 {
 	//Display the trackview for the activated playlist
 
-        int pos = index.data(Qt::UserRole).toInt();
+        int pos = index.data(ListViewDelegate::customDataRole).toInt();
 
-        Spotify::Playlist* pPlaylist = m_Playlists.at(pos);
-
-        if(m_pTrackView)
+        if(pos!=-1) //-1 in case offline mode is active, and the particular list has not been synced offline.
         {
-            if(m_pQueueView)
-            {
-                if(m_pQueueView->parent()!=m_pTrackView)
-                    m_pQueueView->setParent(m_pTrackView,m_pQueueView->windowFlags());//re-parent QueueView instance (to TrackView)
-            }
-            m_pTrackView->setPlaylist(pPlaylist); //set active playlist for track view
-            m_pTrackView->show();
-        }
+            Spotify::Playlist* pPlaylist = m_Playlists.at(pos);
 
+            if(m_pTrackView)
+            {
+                if(m_pQueueView)
+                {
+                    if(m_pQueueView->parent()!=m_pTrackView)
+                        m_pQueueView->setParent(m_pTrackView,m_pQueueView->windowFlags());//re-parent QueueView instance (to TrackView)
+                }
+                m_pTrackView->setPlaylist(pPlaylist); //set active playlist for track view
+                m_pTrackView->show();
+            }
+        }
 }
 
 void MainWindow::OnMainMenuAction(QAction* action)
 {
-
   //User has selected an item from the app main menu
 
-    if(action->text()==tr("Settings"))
+    if(action->data().toInt()==MENU_ACTION_SETTINGS)
     { //display the app Settings dialog
         //display the Settings dialog
         m_pSettingsDlg->readSettings();
         m_pSettingsDlg->showMaximized();
     }
-    else if(action->text()==tr("FM Transmitter"))
+    else if(action->data().toInt()==MENU_ACTION_FM_TRANSMITTER)
     { //display the FM transmitter control panel dialog (plugin)
         OnDisplayFMTransmitterDialog();
     }
-    else if(action->text()==tr("Online Help"))
-    { //open / display online help content
-        m_pHelpViewer->setContent("http://qspot.garage.maemo.org/qspot_help");
+    else if(action->data().toInt()==MENU_ACTION_ONLINE_HELP)
+    { //open / display online help contenttp://qspot.garage.maemo.org/qspot_hel
+        m_pHelpViewer->setContent("htp");
         m_pHelpViewer->showMaximized();
     }
-    else if(action->text()==tr("About"))
+    else if(action->data().toInt()==MENU_ACTION_ABOUT)
     { //display the app About dialog
-        m_AboutDlg.exec();
+        showAboutDialog();
     }
-    else if(action->text()==tr("Log Out"))
-    { //log out the current user (and forget saved credentials)
+    else if(action->data().toInt()==MENU_ACTION_LOGOUT)
+    { //log out the current user (and remove any saved credentials)
         logout();
     }
 }
@@ -848,14 +925,14 @@ void MainWindow::OnScrobblingStatus(bool status, const QString& error)
 void MainWindow::OnScrobbleUpdateNowPlaying(const QString& strTrackName, const QString& strArtistName, const QString& strAlbumName, const QString& strAlbumArtist)
 {
     //update Last.fm now playing info
-    if(m_pAppSettings->value("QSpot/enableScrobbling",ENABLE_SCROBBLING_DEFAULT).toInt()) //scrobbling enabled
+    if(m_LastfmScrobblingEnabled) //scrobbling enabled
         m_pLastFMScrobbler->updateNowPlaying(strTrackName,strArtistName,strAlbumName,strAlbumArtist);
 }
 
 void MainWindow::OnScrobbleTrack()
 {
     //scrobble current track to Last.fm (and send current queue of scrobbles to Last.fm)
-     if(m_pAppSettings->value("QSpot/enableScrobbling",ENABLE_SCROBBLING_DEFAULT).toInt()) //scrobbling enabled
+     if(m_LastfmScrobblingEnabled) //scrobbling enabled
         m_pLastFMScrobbler->scrobbleTrack();
 }
 
@@ -928,41 +1005,148 @@ void MainWindow::OnHWZoomKeyPressed(quint32 keycode)
 void MainWindow::OnLoggedOut()
 {
     //logged out notification for current user
-    Spotify::Error err;
-
-    if(m_reConnectReq)
-    {
-        qDebug() << "Logged out, reconnecting to Spotify..";
-        m_pSession->relogin(); //try to re-connect to Spotify using current session credentials
-    }
-    else
-    {
-        close(); //close down application on explicit logout request
-    }
-}
-
-void MainWindow::OnSpotifyReconnectRequired()
-{
-    //notification that current connection state has changed; reconnection to Spotify backend is required.
-
-    qDebug() << "Reconnect req received..";
-    if(!m_reConnectReq)
-    {
-        m_reConnectReq = true;
-        m_pSession->logout(); //logout the current user, before initiating new login
-    }
+    close(); //close down application on explicit logout request
 }
 
 void MainWindow::OnContextMenuRequested(QPoint pnt)
 {
-    //Start offline sync. for selected playlist (long press on playlist)
-    Q_UNUSED(pnt);
+    //display context menu for marking / un-marking playlist for offline mode
+    QMenu ctxMenu;
+    QString strActionTxt;
 
-    Spotify::Playlist* pPlaylist = m_Playlists.at(m_pPlaylistView->currentIndex().row());
-
-    if(pPlaylist)
+    if(m_pPlaylistView->currentIndex().row()>=0)
     {
-         qDebug() << "Starting offline sync of playlist:" << pPlaylist->getName();
-        m_pSession->setPlaylistOfflineMode(pPlaylist,true);
+        Spotify::Playlist* pPlaylist = m_Playlists.at(m_pPlaylistView->currentIndex().row());
+        if(pPlaylist)
+        {
+            if(pPlaylist->getOfflineStatus()!=Spotify::Available)
+            {
+                strActionTxt = tr("Set available offline");
+            }
+            else
+            {
+                strActionTxt = tr("Unset available offline");
+            }
+            ctxMenu.addAction(strActionTxt);
+            if(pPlaylist->getOfflineStatus()==Spotify::Available||!m_pSession->isOfflineMode())
+            {//only show the context menu in case we are online, or in case the activated playlist is available offline
+                if(ctxMenu.exec(pnt))
+                {//set / unset offline mode for playlist depending on its current offline status
+                    m_pSession->setPlaylistOfflineMode(pPlaylist,pPlaylist->getOfflineStatus()!=Spotify::Available);
+                    if(pPlaylist->getOfflineStatus()!=Spotify::Available)
+                        m_pSyncProgress->setVisible(true); //display progressbar
+                }
+            }
+        }
     }
 }
+
+void MainWindow::OnOfflineSyncStatusUpdated(bool newSync, qint32 progress)
+{
+    //invoked when offline sync status is updated for update of operation progress (first param. specifies if new sync. operation; second param. progress in percentage)
+
+    if(newSync)
+    {
+        //new sync operation started; display user notification and show the progress indicator
+        if(!m_pSyncProgress->isVisible())
+            m_pSyncProgress->setVisible(true);
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this,tr("Offline synchronisation of playlists in progress"));
+#else
+        Banner::information(this,tr("Offline synchronisation of playlists in progress"));
+#endif
+    }
+
+    if(progress==m_pSyncProgress->maximum())
+    {//hide progress bar when sync completed
+        m_pSyncProgress->setVisible(false);
+        m_pSyncProgress->setValue(0);
+    }
+    else
+    { //update progressbar state
+        m_pSyncProgress->setValue(progress);
+    }
+}
+
+void MainWindow::OnOfflineSyncError(const QString& strError)
+{
+    //Notification about offline sync. error. Display notification to user.
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this,tr("Error: ") + strError);
+#else
+        Banner::information(this,tr("Error: ") + strError);
+#endif
+}
+
+void MainWindow::OnOfflineModeStateChanged(bool offline)
+{
+    //Notification received when current oflline state has changed (e.g., online -> offline).
+    //The parameter online indicates new offline state.
+
+    bool forceShowMainView = false; //true in case the control should be force to the main view
+
+    updatePlaylistModel(); //update playlists model state
+
+    //enable / disble search based on new state
+    m_SearchInput.setEnabled(!offline);
+    m_SearchBtn.setEnabled(!offline);
+
+    if(m_LastfmScrobblingEnabled)
+    {
+        //Last.fm scrobbling active; set offline mode on / off
+        m_pLastFMScrobbler->setOfflineMode(offline);
+        if(!offline) //authenticate new session
+            m_pLastFMScrobbler->authenticateSession(m_pAppSettings->value("QSpot/lastfmUid","").toString(),m_pAppSettings->value("QSpot/lastfmPwd","").toString());
+    }
+
+    m_pOnlineStateAction->setChecked(offline);
+
+    if(offline)
+    { //offline mode activated
+        if(!m_pQueueView->isQueueOffline())
+        { //tracks in current playqueue are not available offline
+            if(m_pQueueView->isVisible())
+                m_pQueueView->hide();
+            m_PlayQueue.clear(); //clear playqueue list
+            OnPlayQueuePosUpdated(-1); //clear now playing infobar
+            forceShowMainView = true;
+        }
+        if(m_pTrackView->isVisible())
+        {
+            if(m_pTrackView->getActivePlaylist())
+            {
+                if(m_pTrackView->getActivePlaylist()->getOfflineStatus()!=Spotify::Available) //current playlist is not available offline
+                {
+                    m_pTrackView->hide();
+                    forceShowMainView = true;
+                }
+            }
+            else
+            {
+                m_pTrackView->hide();
+                forceShowMainView = true;
+            }
+        }
+        if(forceShowMainView)
+            this->show();
+#ifdef Q_WS_MAEMO_5
+        QMaemo5InformationBox::information(this,tr("Offline mode is now activated"));
+#else
+        Banner::information(this,tr("Offline mode is now activated"));
+#endif
+    }
+}
+
+void MainWindow::OnOfflineModeStateToggled(bool checked)
+{
+    //The user has manually toggled between offline / online states in UI.
+    //In case checked parameter is true online -> offline has been toggled; otherwise offline -> online.
+    m_pSession->setOfflineMode(checked,true);
+
+    //remember the current user setting
+    if(checked)
+        m_pAppSettings->setValue("QSpot/OfflineMode",FORCE_OFFLINE_MODE_ON);
+    else
+        m_pAppSettings->setValue("QSpot/OfflineMode",FORCE_OFFLINE_MODE_OFF);
+
+}
