@@ -19,10 +19,10 @@
 #include <QtGui/QPushButton>
 #include <QtGui/QKeyEvent>
 #include <QtGui/QCloseEvent>
+#include <QtGui/QProgressBar>
 #include <QSettings>
 #include <QMessageBox>
 #include <QFont>
-#include <QTimer>
 #include <QTranslator>
 #ifdef Q_WS_MAEMO_5
     #include <QtMaemo5/QMaemo5InformationBox>
@@ -43,14 +43,21 @@
 #include "OssoWrapper.h"
 #include "lastfmscrobbler.h"
 #include "DBusWrapper.h"
-#include "ConnectionManager.h"
 #include "global_defs.h"
 #include "HelpViewer.h"
 
 
 //defines
 
-#define APP_VERSION "0.9.9-4" //application version string
+#define APP_VERSION "1.0.0-1" //application version string
+
+//menu bar actions
+#define MENU_ACTION_SETTINGS        0
+#define MENU_ACTION_FM_TRANSMITTER  1
+#define MENU_ACTION_ONLINE_HELP     2
+#define MENU_ACTION_ABOUT           3
+#define MENU_ACTION_LOGOUT          4
+#define MENU_ACTION_OFFLINE_MODE    5
 
 class MainWindow : public QMainWindow
 {
@@ -70,6 +77,10 @@ private:
 
     QSettings* m_pAppSettings; //application settings instance
 
+    QMenu* m_pCtxMenu; //context menu (options for i.e., offline synchronisation)
+
+    QProgressBar *m_pSyncProgress; //progressbar used to display progress of offline sync operation
+
     QWidget m_MainLayout;
 
     QGridLayout m_GridLayout;
@@ -96,8 +107,6 @@ private:
 
     DBusWrapper* m_pDBusWrapper; //wrapper instance for D-bus interface
 
-    ConnectionManager* m_pConnMan; //connection manager instance pointer (handles e.g., online state change  / roaming notifications)
-
     LastFMScrobbler* m_pLastFMScrobbler; //Last.fm scrobbling instance
 
     QPushButton m_SearchBtn;
@@ -134,27 +143,32 @@ private:
 
     bool m_update; //TRUE in case UI elements should be updated
 
+    bool m_LastfmScrobblingEnabled; //TRUE in case Last.fm scrobbling enabled
+
     qint64 m_currPlaybackPos; //playback position for currently playing track
 
     AboutDialog m_AboutDlg; //App About dialog instance
 
-    QTimer *m_pTimeoutTimer; //timer instance used for detecting timeouts (i.e., during playlist loading)
-
-    QTimer *m_pReconnectTimer; //timer instance used for re-connect attempts
-
-    bool m_SessionInit; //TRUE if Spotify session has been initialised (needs to be initialised before loggin in)
-
-    bool m_AudioInit; //TRUE in case audio playback thread initialised
-
     quint32 m_currSysVolume; //current system volume (used for volume control using HW zoom keys when screen is in locked state)
 
     qint32 m_currOrientation; //current display orientation
 
-    bool m_reConnectReq; //TRUE in case re-connect to Spotify has been requested (used to prevent playlist metadata etc. to be reloaded in case connection is lost to Spotify backend)
+    QAction* m_pSettingsAction; //action associated with Settings menu entry
+
+    QAction* m_pAboutAction; //action associated with About menu entry
+
+    QAction* m_pHelpAction; //action associated with Help menu entry
+#ifdef Q_WS_MAEMO_5
+    QAction* m_pFMTransAction; //action associated with FM Transmitter menu entry
+#endif
+
+    QAction* m_pLogoutAction; //action associated with the Log Out menu entry
+
+    QAction* m_pOnlineStateAction; //action associated with the Go Offline / Go Online menu entry
 
     void logout(); //logout current user and display the login dialog
 
-    void initAboutDialog(); //setup the application about dialog
+    void showAboutDialog(); //display the application about dialog
 
     void updateSettings();  //update / set application-related settings
 
@@ -164,6 +178,8 @@ private:
 
     void setupUILang(); //setup the UI language
 
+    void updatePlaylistModel(); //update the listmodel from underlying playlist instances
+
 
 private slots:
 
@@ -173,6 +189,9 @@ private slots:
      void OnLoggedOut();
      void OnPlaylistContainersReady(QList<Spotify::Playlist*> playlists);
      void OnPlaylistActivated(QModelIndex index);
+     void OnPlaylistAdded(Spotify::Playlist* playlist, qint32 position); //a playlist has been added
+     void OnPlaylistRemoved(qint32 position); //a playlist has been removed
+     void OnPlaylistUpdated(); //a playlist (e.g., metadata) has been updated
      void OnLoginDialogRejected();
      void OnMainMenuAction(QAction* action);
      void OnPlayingState(bool state);
@@ -184,8 +203,6 @@ private slots:
      void OnPlaybackPositionUpdated(qint64 pos);
      void OnPlaybackTokenLost();
      void OnSettingsSaved();
-     void OnPlaylistLoadingTimeout();
-     void OnReconnectTimeout();
      void OnDisplayFMTransmitterDialog();
      void OnShowAlbumView(Spotify::Album* pAlbum);
      void OnShowArtistView(Spotify::Artist* pArtist);
@@ -197,15 +214,16 @@ private slots:
 #ifdef Q_WS_MAEMO_5
      void OnHWZoomKeyPressed(quint32 keycode); //notification that HW zoom key has been pressed
 #endif
-     void OnSpotifyReconnectRequired(); //notification that current connection state has changed, and it is required to reconnect to the Spotify backend
      void OnSpotifyLogin(const QString& strUserID, const QString& strPassword, bool saveCredentials); //slot for handling login to the Spotify backend service
      void OnContextMenuRequested(QPoint pnt);
+     void OnOfflineSyncStatusUpdated(bool newSync, qint32 progress); //invoked when offline sync status is updated for update of operation progress (first param. indicates whether new sync operation; second param progress in percentage)
+     void OnOfflineModeStateChanged(bool offline); //notification received when current offline state has changed (e.g., online -> offline)
+     void OnOfflineSyncError(const QString& strError); //notification about offline sync. error
+     void OnOfflineModeStateToggled(bool checked); //user has manually toggled between offline / online states
 
 signals:
 
      void showLogin();
-     void handleLoginType();
-
 };
 
 #endif // MAINWINDOW_H
