@@ -1,8 +1,12 @@
+/**
+ *  spotifysession.h: Spotify session class. Handles the session functions.
+ */
 #ifndef SPOTIFYSESSION_H
 #define SPOTIFYSESSION_H
 
 #include <libspotify/api.h>
 #include <QMetaType>
+#include <QDebug>
 #include <QList>
 #include <QImage>
 #include <QHash>
@@ -20,9 +24,11 @@
 #include "albumbrowser.h"
 #include "search.h"
 #include "user.h"
+#include "link.h"
+#include "artist.h"
+#include "album.h"
 #include "PlaybackThread.h"
-
-
+#include "connectionmanager.h"
 
 namespace Spotify
 {
@@ -30,21 +36,28 @@ namespace Spotify
     class Session: public QObject
     {
         Q_OBJECT
-        public:
 
+    public:
 
             Session( QObject* parent = 0 );
             virtual ~Session();
 
-            void init(const sp_bitrate& bitrate = SP_BITRATE_160k); //init a new Spotify session (optional parameter the preferred streaming bitrate)
+            /**
+             * Pointer to the Session object. Used to map
+             * C-callbacks to C++ members.
+             */
+            static Spotify::Session* callingSession;
+
+            // For mapping sp_playlist pointers to corresponding Spotify::Playlist pointers
+            QHash< sp_playlist*, Spotify::Playlist* > m_playlistHash;
+
+            void init(); //initialise a new Spotify session
 
             void initAudio(); //init the audio / playback thread
 
-            void setPreferredBitrate(const sp_bitrate& bitrate); //set preferred audio playback bitrate
+            void setPreferredBitrate(const sp_bitrate& bitrate); //set preferred audio streaming bitrate
 
-            bool playlistLoadingInProgress(); //returns true in case playlist loading (getPlaylists) is currently in progress
-
-            void stopPlaylistLoading(); //request to stop the playlist loading (getPlaylists).
+            void setPreferredSyncBitrate(const sp_bitrate& bitrate, bool resyncTracks = false); //set preferred offline synchronisation bitrate (resync existing tracks in case resyncTracks is true)
 
             void setMinBufferSize(qint32 secs); //set the size of the internal FIFO queue used to hold audio data
 
@@ -68,20 +81,19 @@ namespace Spotify
 
             void resetPlaybackMembers();  //reset the playback related instance members (and removes any pending events from the playback thread)
 
-            QString getRememberedUser(); //Returns remembered UserID for the session
-            QString getUserPwd(); //Returns remembered password for the session
+            bool isOfflineMode(); //returns true in case logged in offline; false in case we are currently online and connected to Spotify access point.
 
-            /**
-             * Pointer to the Session object. Used to map
-             * C-callbacks to C++ members.
-             */
-            static Spotify::Session* callingSession;
+            QString getRememberedUser(); //Returns remembered (logged-in) UserID for the session
 
-            /**
-             * Check if the session is valid.
-             * @return True if the session is valid, false if it is not.
-             */
-            bool isValid() { return m_valid; }
+            QString getCurrentUser(); //Returns the currently username for the currently logged-in user
+
+            bool getOfflineTimeLeft(qint32& days, qint32& hours, qint32& minutes); //Returns time until the offline keystore expires and user is required to go online (days, hours, and minutes
+                                                                                   //left as ref. params). Returns true on success; false on error.
+
+            void setAllowMobileSync(bool allow); //Set whether offline syncing is allowed over mobile (e.g., 2G/3G) connections
+
+            void setOfflineMode(bool offline, bool userRequested); //Toggle between offline / online playback mode (according to offline param). In case state change is requested by user,
+                                                                   //userRequested parameter is set to true; otherwise false (requested by system i.e., due to changed online connectivity state).
 
 
             /**
@@ -90,16 +102,8 @@ namespace Spotify
              */
             Spotify::User* getUser() const { return m_currentUser; }
 
-
             /**
-             * Request (root) playlist container (contains all playlists)
-             */
-
-            void getPlaylistContainer();
-
-
-            /**
-             * Logs a user in to spotify.
+             * Handles login of specified user.
              * @param username Username
              * @param password Password
              * @param storeCredentals Whether user credentials should be remembered
@@ -163,11 +167,6 @@ namespace Spotify
              */
             bool loadImage(const byte* imageID, Spotify::ImageContainer* pContainer);
 
-            /**
-             * Get the collection of playlists.
-             */
-            Spotify::Error getPlaylists();
-
             /**
              * Add a new playlist to the playlist container by name.
              * @param name Name of the new playlist.
@@ -222,7 +221,7 @@ namespace Spotify
             bool browse( Spotify::Album* album, qint32* pCallerID);
 
 
-            //libopenspotify callbacks
+            //libspotify callbacks
 
             // Artist Browsing callbacks:
             static void SP_CALLCONV artistBrowseCompleteCallback( sp_artistbrowse* ab, void* userdata );
@@ -242,27 +241,8 @@ namespace Spotify
                                                int oldPosition, int newPosition, void* userdata );
 
 
-            // Playlist callbacks:
-            static void SP_CALLCONV tracksAddedCallback( sp_playlist* pl, sp_track* const* tracks, int num_tracks,
-                                            int position, void* userdata );
-
-
-            static void SP_CALLCONV tracksRemovedCallback( sp_playlist* pl, const int* tracks, int num_tracks, void* userdata );
-
-            static void SP_CALLCONV tracksMovedCallback( sp_playlist* pl, const int* tracks, int num_tracks, int new_position,
-                                             void* userdata );
-
             static void SP_CALLCONV playListContainerLoadedCallback(sp_playlistcontainer *pc, void* userdata);
 
-
-            static void SP_CALLCONV playlistRenamedCallback( sp_playlist* pl, void* userdata );
-
-            static void SP_CALLCONV playlistStateChangedCallback( sp_playlist* pl, void* userdata );
-
-            static void SP_CALLCONV playlistUpdateInProgressCallback( sp_playlist* pl, bool done, void* userdata );
-
-            static void SP_CALLCONV playlistMetadataUpdatedCallback( sp_playlist* pl, void* userdata );
-
             // Image callbacks:
             static void SP_CALLCONV imageLoadedCallback( sp_image* image, void* userdata );
 
@@ -271,32 +251,24 @@ namespace Spotify
 
             // Session callbacks:
             static void SP_CALLCONV loggedInCallback( sp_session* session, sp_error error );
-
             static void SP_CALLCONV loggedOutCallback( sp_session* session );
-
             static void SP_CALLCONV metadataUpdatedCallback( sp_session* session );
-
             static void SP_CALLCONV connectionErrorCallback( sp_session* session, sp_error error );
-
             static void SP_CALLCONV  messageToUserCallback( sp_session* session, const char* message );
-
             static void SP_CALLCONV notifyMainThreadCallback( sp_session* session );
-
             static int SP_CALLCONV musicDeliveryCallback( sp_session* session, const sp_audioformat* format,
                                                const void* frames, int num_frames );
-
             static void SP_CALLCONV playTokenLostCallback( sp_session* session );
-
             static void SP_CALLCONV logMessageCallback( sp_session* session, const char* message );
-
             static void SP_CALLCONV endOfTrackCallback( sp_session* session );
-
             static void SP_CALLCONV streamingError( sp_session* session, sp_error error );
-
             static void SP_CALLCONV offlineStatusUpdated( sp_session* session);
+            static void SP_CALLCONV offlineSynchronistationError( sp_session* session, sp_error error);
 
-
-        public slots:
+            /**
+              * Forget the currently logged-in user's credentials (in case stored by libspotify)
+             */
+            void forgetCurrentUser();
 
             /**
              * Log the current user out of Spotify.
@@ -304,16 +276,15 @@ namespace Spotify
             void logout();
 
         private slots:
-            void updatePlaylistsMetadataSlot();
+
+            void updateMetadata(); //metadata has been updated (for e.g., playlists)
             void searchDeletedSlot( QObject* parent );
             void artistCatalogueDeletedSlot( QObject* parent );
             void albumBrowserDeletedSlot( QObject* parent );
             void OnProcessThreadEvents();
             void OnRootListContainerLoaded(sp_playlistcontainer* pc);
-            void OnPlayListAdded(sp_playlist* playlist, int position);
-            void OnPlayListRenamed(sp_playlist* playlist);
-            void OnTracksAdded(sp_playlist* pl, sp_track* const* tracks, int num_tracks, int position);
-            void OnPlaylistMetadataUpdated(sp_playlist* pl);
+            void OnPlayListAdded(sp_playlist* playlist , qint32 position);
+            void OnPlayListRemoved(sp_playlist* playlist, qint32 position);
             void OnImageLoaded(sp_image* image, void* userdata);
             void OnMusicDelivery(int num_frames, const sp_audioformat* format, QByteArray buffer);
             void OnSearchComplete(sp_search* search);
@@ -323,6 +294,8 @@ namespace Spotify
             void OnPlaybackPositionUpdated(qint64 duration);
             void OnPlaybackFinished();
             void OnPlayingState(bool state);
+            void OnOfflineStatusUpdated();
+            void OnOnlineStateChanged(bool online);
 
         signals:
             /**
@@ -382,7 +355,13 @@ namespace Spotify
              * Signal emitted when a playlist has been added / loaded to playlist container at specified position (internal usage)
              */
 
-            void onPlaylistAdded(sp_playlist*, int);
+            void onPlaylistAdded(sp_playlist*, qint32);
+
+            /**
+             * Signal emitted when a playlist has been removed from playlist container at specified position (internal usage)
+             */
+
+            void onPlaylistRemoved(sp_playlist*, qint32);
 
             /**
              * Signal emitted when a playlist has been renamed (internal usage)
@@ -396,11 +375,21 @@ namespace Spotify
 
             void onTracksAdded(sp_playlist*, sp_track* const*, int, int);
 
-	     /**
+             /**
              * Signal emitted when image data has been loaded (e.g., album coverart)
              */
             void onImageLoaded(sp_image*, void*); 
 
+            /**
+            * Signal emitted when offline status has been updated
+            */
+            void onOfflineStatusUpdated();
+
+            /**
+            * Signal emitted when offline synchronisation error status have been updated (parameter contains error message)
+            */
+            void offlineSyncError(const QString&);
+
             /**
              * Signal emitted when playlist containers have been received. The Spotify::Session object
              * owns the Spotify::Playlist data and the Spotify::Track data within and will delete
@@ -418,10 +407,10 @@ namespace Spotify
              * Signal emitted when a new playlist has been added. The Spotify::Session object
              * owns the Spotify::Playlist data and the Spotify::Track data within and will delete
              * these when destructed.
-             * @playlist The playlist that has been added.
+             * @playlist Playlist added.
              * @position The index of the new playlist.
              */
-            void playlistAdded( Spotify::Playlist* playlist, int position );
+            void playlistAdded( Spotify::Playlist* playlist, qint32 position );
 
 
             /**
@@ -435,7 +424,7 @@ namespace Spotify
              * Signal emitted when a playlist has been removed.
              * @param position The index of the playlist that has been removed.
              */
-            void playlistRemoved( int position );
+            void playlistRemoved( qint32 position );
 
             /**
              * Signal emitted when playback has finished.
@@ -453,7 +442,7 @@ namespace Spotify
              * Signal emitted when the current buffering amount has been updated.
              * @param bufferedAmout The amount buffered of the currently loaded track (in milliseconds).
              */
-            void trackBufferingAmountUpdated( qint32 bufferedAmount );
+            void trackBufferingAmountUpdated( quint32 bufferedAmount );
 
             /**
              * Signal emitted when the current playback position in the track has been updated.
@@ -484,45 +473,80 @@ namespace Spotify
 
             void bufferNextTrack();
 
+            /**
+             * Signal emitted when offline sync status updated and sync is in progress (first param indicates if it is new sync operation; second parameter gives percentage completed)
+             */
+            void offlineSyncStatusUpdated(bool, qint32);
+
+            /**
+             * Signal emitted when current offline state has changed, e.g., online -> offline (boolean specifies new state; true = offline mode active).
+             */
+            void offlineModeStateChanged(bool);
+
 
         private:
 
             Spotify::Error m_error;
             Spotify::Track* m_currentTrack;
 
+            ConnectionManager* m_pConnMan; //connection mananger instance (get current online status etc.)
+
             sp_session_config m_config;
-            sp_session* m_session;
+            sp_session* m_session; //current session instance
             sp_playlistcontainer* m_playlistcontainer;
-
-            bool m_valid;
-            QString m_username;
-            QString m_password;
             Spotify::User* m_currentUser;
+            sp_connection_rules m_connRules; //currently set connection rules
+            sp_connectionstate m_connState; //current session connection status
 
-            qint32 m_bufferedAmount; //current buffered amount (of the currently loaded track)
+            QByteArray m_audioFragmentBuffer; //temporary buffer that holds enough samples (i.e., one sec of audio); the buffer is then appended to main playback thread FIFO.
+            qint32 m_framesConsumed; //number of samples currently held in the temporary audio buffer
+
+            bool m_allPlaylistsLoaded; //TRUE in case metadata for all playlists in the root container has been loaded; false otherwise.
+
+            quint32 m_bufferedAmount; //current buffered amount (of the currently loaded track)
             qint64 m_playbackPos; //current playback position for the currently loaded track
 
             bool m_buffering; //true in case we are currently buffering
-
-            bool m_ProcessEvents;
-
-            bool m_bPlaylistsLoading; //TRUE in case playlist loading is in progress
-
-            bool m_bStopPlaylistLoading; //TRUE in case playlist loading should be stopped (interrupted)
-
-            bool m_DisableNotifyThread;
-
+            bool m_ProcessEvents; //set to true when events in the internal libspotify threads should be processed
             bool m_preBufActive; //true if track pre-buffering is enabled
-
             QTimer *m_pProcEvtTimer;
+            bool m_offlineSyncing;  //set to true in case offline sync is in progress; false otherwise
+            bool m_offlineMode;  //true in case offline mode is active; false otherwise
+            bool m_offlineModeForced; //true in case offline mode has been forced by the user (i.e., not automatically); false otherwise
 
             /**
              * Audio playback thread
              */
             PlaybackThread* m_pAudioThread;
 
-            // For mapping sp_playlist pointers to Spotify::Playlist pointers
-            QHash< sp_playlist*, Spotify::Playlist* > m_playlistHash;
+            /**
+             * Update the current session connection status
+             * @return void
+             */
+             void updateConnectionState();
+
+            /**
+             * Set the connection rule specified to state (on / off)
+             * @param rule Rule to set / un-set
+             * @param state Rule state (true = set / false = un-set)
+             * @return void
+             */
+            void setConnectionRule(sp_connection_rules rule, bool state);
+
+            /**
+             * Set  the current type of connection (e.g., wifi / 3G etc.)
+             * @param type Current connection type
+             * @return void
+             */
+            void setConnectionType(sp_connection_type type);
+
+            /**
+             *  Set connection type / rules
+             * @return void
+             */
+            void setNetworkState();
+
+            //sets connection type / rules
 
             // For maintaing a sorted (by index) list of Spotify::Playlist pointers
             QList< Spotify::Playlist* > m_playlistList;
@@ -554,25 +578,10 @@ namespace Spotify
             void playlistMovedWrapper( sp_playlistcontainer* pc, sp_playlist* playlist,
                                        int oldPosition, int newPosition, void* userdata );
 
-            void tracksAddedWrapper( sp_playlist* pl, sp_track* const* tracks, int num_tracks,
-                                    int position, void* userdata );
-
-            void tracksRemovedWrapper( sp_playlist* pl, const int* tracks, int num_tracks, void* userdata );
-
-            void tracksMovedWrapper( sp_playlist* pl, const int* tracks, int num_tracks, int new_position,
-                                     void* userdata );
 
             void playListContainerLoadedWrapper(sp_playlistcontainer *pc, void* userdata);
 
 
-            void playlistRenamedWrapper( sp_playlist* pl, void* userdata );
-
-            void playlistStateChangedWrapper( sp_playlist* pl, void*  userdata );
-
-            void playlistUpdateInProgressWrapper( sp_playlist* pl, bool done, void* userdata );
-
-            void playlistMetadataUpdatedWrapper( sp_playlist* pl, void* userdata );
-
             void imageLoadedWrapper( sp_image* image, void* userdata );
 
 
@@ -602,10 +611,7 @@ namespace Spotify
 
             void offlineStatusUpdatedWrapper();
 
-            /* returns true in case all playlist containers have been completely loaded and added to internal playlists list */
-            bool isPlaylistContainersReady();
-
-
+            void offlineSynchronisationErrorWrapper(sp_error error);
     };
 }
 
