@@ -62,10 +62,9 @@ namespace Spotify
     {
         for( int i = 0; i < tracks.count(); ++i )
         {
-            m_tracks.insert( position+i, tracks.at(i) );
-            m_tracksOriginal.insert( position+i, tracks.at(i) );
-            qDebug() << "Track inserted";
-            emit trackAdded( tracks.at(i), position+i );
+            m_tracks.insert( position+1, tracks.at(i) );
+            m_tracksOriginal.insert( position+1, tracks.at(i) );
+            emit trackAdded( tracks.at(i), position+1 );
         }
         emit updated();
     }
@@ -80,13 +79,14 @@ namespace Spotify
     void TrackCollection::removeTracks( const int* positions, int num_tracks )
     {
         // Remove tracks backwards, so we don't screw up the indices of the other items:
-        for( int i = num_tracks-1; i >= 0; --i )
+       /* for( int i = num_tracks-1; i >= 0; --i )
         {
             Spotify::Track* track = m_tracks.takeAt( positions[i] );
             m_tracksOriginal.removeAt( positions[i] );
             emit trackRemoved( positions[i] );
             track->deleteLater();
         }
+        */
         emit updated();
     }
 
